# TPK4186 - Advanced Tools for Performance Engineering
# XMLInterface.py
# by Antoine Rauzy
# Copyrights (c) 2019-2020 NTNU
#
# This file is part of the educational material provided to NTNU students registered to the course TPK4186.
# This work is licensed under a Creative Commons Attribution-NonCommercial 4.0 International License.
# For details about this license, see https://creativecommons.org/licenses/by-nc/4.0/.

# Table of Contents
# -----------------
# 1. Required modules
# 2. XML Printer
# 3. XML Parser

# 1. Required modules
# -------------------

from Retailer import *
import xml.dom.minidom

# 2. XML Printer
# --------------

class XMLPrinter:

  def ExportRetailerSalesAtXMLFormat(self, retailer, fileName):
    xmlImplementation = xml.dom.minidom.getDOMImplementation()
    xmlDocument = xmlImplementation.createDocument(None, "retailerSales", None)
    self.UploadRetailerSalesInDocument(retailer, xmlDocument)
    try:
      outputFile = open(fileName, "w")
    except:
      sys.stderr.write('Unable to open file "%s"\n' % fileName)
      sys.stderr.flush()
      return 1
    outputFile.write(xmlDocument.toprettyxml("\t"))
    outputFile.flush()
    outputFile.close()
    return 0

  def UploadRetailerSalesInDocument(self, retailer, xmlDocument):
    xmlRootElement = xmlDocument.documentElement
    for product in retailer.GetProducts():
      self.UploadProductSalesInDocument(product, xmlDocument, xmlRootElement)

  def UploadProductSalesInDocument(self, product, xmlDocument, xmlParentElement):
    xmlProductElement = xmlDocument.createElement("product")
    xmlParentElement.appendChild(xmlProductElement)
    xmlProductElement.setAttribute("reference", product.GetName())
    xmlChildElement = xmlDocument.createElement("profitPerSale")
    xmlProductElement.appendChild(xmlChildElement)
    xmlChildElement.setAttribute("value", "{0:.2f}".format(product.GetProfitPerSale()))
    xmlSalesElement = xmlDocument.createElement("sales")
    xmlProductElement.appendChild(xmlSalesElement)
    for monthNumber in range(1, 13):
      xmlChildElement = xmlDocument.createElement("month")
      xmlSalesElement.appendChild(xmlChildElement)
      xmlChildElement.setAttribute("trigram", self.GetMonthTrigramFromMonthNumber(monthNumber))
      xmlChildElement.setAttribute("quantity", str(product.GetMonthlySales(monthNumber)))

  def GetMonthTrigramFromMonthNumber(self, number):
    if number==1:
      return "Jan"
    if number==2:
      return "Feb"
    if number==3:
      return "Mar"
    if number==4:
      return "Apr"
    if number==5:
      return "May"
    if number==6:
      return "Jun"
    if number==7:
      return "Jul"
    if number==8:
      return "Aug"
    if number==9:
      return "Sep"
    if number==10:
      return "Oct"
    if number==11:
      return "Nov"
    if number==12:
      return "Dec"
    return None

# 3. XML Parser
# -------------

class XMLParser:

  def ImportRetailerSalesAtXMLFormat(self, retailer, fileName):
    try:
      xmlDocument = xml.dom.minidom.parse(fileName)
    except:
      sys.stderr.write('Error while loading file "%s"\n' % fileName)
      sys.stderr.flush()
      return 1
    error = self.DownloadRetailerSales(retailer, xmlDocument)
    return error

  def DownloadRetailerSales(self, retailer, xmlDocument):
    xmlRootElement = xmlDocument.documentElement
    if xmlRootElement.tagName!="retailerSales":
      sys.stderr.write('An element "retailerSales" was expected\n')
      sys.stderr.flush()
      return 1
    xmlNode = xmlRootElement.firstChild
    error = 0
    while xmlNode!=None:
      if xmlNode.nodeType==xml.dom.minidom.Node.ELEMENT_NODE and xmlNode.tagName=="product":
        error = self.DownloadProduct(retailer, xmlNode)
      if error!=0:
        break
      xmlNode = xmlNode.nextSibling
    return error

  def DownloadProduct(self, retailer, xmlElement):
    productName = xmlElement.getAttribute("reference")
    if productName==None:
      sys.stderr.write('An attribute "reference" was expected\n')
      sys.stderr.flush()
      return 1
    product = retailer.NewProduct(productName)
    xmlNode = xmlElement.firstChild
    error = 0
    while xmlNode!=None:
      if xmlNode.nodeType==xml.dom.minidom.Node.ELEMENT_NODE and xmlNode.tagName=="profitPerSale":
        error = self.DownloadProfitPerSale(product, xmlNode)
      elif xmlNode.nodeType==xml.dom.minidom.Node.ELEMENT_NODE and xmlNode.tagName=="sales":
        error = self.DownloadSales(product, xmlNode)
      if error!=0:
        break
      xmlNode = xmlNode.nextSibling
    return error
 
  def DownloadProfitPerSale(self, product, xmlElement):
    profitPerSale = xmlElement.getAttribute("value")
    if profitPerSale==None or not self.IsFloat(profitPerSale):
      sys.stderr.write('An attribute "value" was expected\n')
      sys.stderr.flush()
      return 1
    product.SetProfitPerSale(float(profitPerSale))
    return 0

  def DownloadSales(self, product, xmlElement):
    xmlNode = xmlElement.firstChild
    error = 0
    while xmlNode!=None:
      if xmlNode.nodeType==xml.dom.minidom.Node.ELEMENT_NODE and xmlNode.tagName=="month":
        error = self.DownloadMonthlySales(product, xmlNode)
      if error!=0:
        break
      xmlNode = xmlNode.nextSibling
    return error

  def DownloadMonthlySales(self, product, xmlElement):
    trigram = xmlElement.getAttribute("trigram")
    if trigram==None:
      sys.stderr.write('An attribute "trigram" was expected\n')
      sys.stderr.flush()
      return 1
    quantity = xmlElement.getAttribute("quantity")
    if quantity==None or not self.IsInteger(quantity):
      sys.stderr.write('An attribute "quantity" was expected\n')
      sys.stderr.flush()
      return 1
    number = self.GetMonthNumberFromMonthTrigram(trigram)
    product.SetMonthlySales(number, int(quantity))
    return 0

  def IsInteger(self, token):
    return re.match("[0-9]+", token)

  def IsFloat(self, token):
    return re.match("[-+]?(\d+(\.\d*)?|\.\d+)([eE][-+]?\d+)?", token)

  def GetMonthNumberFromMonthTrigram(self, trigram):
    if trigram=="Jan":
      return 1
    if trigram=="Feb":
      return 2
    if trigram=="Mar":
      return 3
    if trigram=="Apr":
      return 4
    if trigram=="May":
      return 5
    if trigram=="Jun":
      return 6
    if trigram=="Jul":
      return 7
    if trigram=="Aug":
      return 8
    if trigram=="Sep":
      return 9
    if trigram=="Oct":
      return 10
    if trigram=="Nov":
      return 11
    if trigram=="Dec":
      return 12
    return None
